@@ -55,7 +55,7 @@ short del;
 /* calling routine to free it.						*/
 /*----------------------------------------------------------------------*/
 
-char *evaluate_expr(oparamptr ops)
+char *evaluate_expr(oparamptr ops, objinstptr pinst)
 {
    if (ops->type != XC_EXPR) return NULL;
    return strdup(ops->parameter.expr);
@@ -207,13 +207,41 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
    stringpart *nextptr = strptr->nextpart;
 
    if (strptr->type == PARAM_START)
-      nextptr = linkstring(thisinst, strptr);
-   else if (strptr->type == PARAM_END)
+      nextptr = linkstring(thisinst, strptr, TRUE);
+   else if (strptr->type == PARAM_END) {
       strptr->nextpart = NULL;
 
+      /* Parameters that have a non-NULL entry in data have	*/
+      /* been promoted from an expression or numerical value	*/
+      /* to a string.  The memory allocated for this string	*/
+      /* should be free'd.					*/
+
+      if (strptr->data.string != (u_char *)NULL) {
+	 freelabel((stringpart *)(strptr->data.string));
+	 strptr->data.string = (u_char *)NULL;
+      }
+   }
    return nextptr;
 }
 
+/*----------------------------------------------------------------------*/
+/* Same as the above routine, but don't recompute expression parameters	*/
+/* when encountered.  Use the previously generated result.		*/
+/*----------------------------------------------------------------------*/
+
+stringpart *nextstringpartnorecompute(stringpart *strptr, objinstptr thisinst)
+{
+   stringpart *nextptr = strptr->nextpart;
+
+   if (strptr->type == PARAM_START)
+      nextptr = linkstring(thisinst, strptr, FALSE);
+   else if (strptr->type == PARAM_END) {
+      strptr->nextpart = NULL;
+      if (strptr->data.string != (u_char *)NULL)
+	 freelabel((stringpart *)(strptr->data.string));
+   }
+   return nextptr;
+}
 
 /*----------------------------------------------------------------------*/
 /* Remove a string part from the string					*/
@@ -248,7 +276,14 @@ stringpart *deletestring(stringpart *dstr, stringpart **strtop, objinstptr thisi
 	    Fprintf(stderr, "Error in deletestring:  Bad parameter %s found\n", key); 
 	 }
 	 else {
-	    ops->parameter.string = dstr->nextpart;
+	    switch(ops->type) {
+	       case XC_STRING:
+		  ops->parameter.string = dstr->nextpart;
+		  break;
+	       default:
+		  /* What to be done here? */
+		  break;
+	    }
 	 }
       }
       /* If this is the end of a parameter, we have to link the		*/
@@ -303,12 +338,17 @@ stringpart *mergestring(stringpart *firststr)
 
 /*----------------------------------------------------------------------*/
 /* Link a parameter to a string						*/
+/* If compute_exprs is TRUE, then we should recompute any expression	*/
+/* parameters encountered.  If FALSE, then we assume that all		*/
+/* expressions have been computed previously, and may use the recorded	*/
+/* instance value.							*/
 /*----------------------------------------------------------------------*/
 
-stringpart *linkstring(objinstptr localinst, stringpart *strstart)
+stringpart *linkstring(objinstptr localinst, stringpart *strstart,
+	Boolean compute_exprs)
 {
    char *key;
-   stringpart *nextptr = NULL, *tmpptr;
+   stringpart *tmpptr, *nextptr = NULL;
    static stringpart *promote = NULL;
    oparamptr ops;
 
@@ -335,23 +375,41 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
 
    if (ops->type != XC_STRING) {
 
-      /* Generate static string for promoting numerical parameters */
       if (promote == NULL) {
-	 tmpptr = makesegment(&promote, NULL);
-	 tmpptr->type = TEXT_STRING;
-	 tmpptr->data.string = (char *)malloc(13);
-	 tmpptr = makesegment(&promote, NULL);
-	 tmpptr->type = PARAM_END;
+         /* Generate static string for promoting numerical parameters */
+         tmpptr = makesegment(&promote, NULL);
+         tmpptr->type = TEXT_STRING;
+         tmpptr = makesegment(&promote, NULL);
+         tmpptr->type = PARAM_END;
       }
+      else
+	 free(promote->data.string);
 
       /* Promote numerical type to string */
-      if (ops->type == XC_INT)
+      if (ops->type == XC_INT) {
+	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%12d", ops->parameter.ivalue);
-      else if (ops->type == XC_FLOAT)
+         nextptr = promote;
+      }
+      else if (ops->type == XC_FLOAT) {
+	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%g", (double)(ops->parameter.fvalue));
-      else
-	 promote->data.string = evaluate_expr(ops);
-      nextptr = promote;
+         nextptr = promote;
+      }
+      else {	/* ops->type == XC_EXPR */
+	 oparamptr ips;
+	 if (!compute_exprs && (ips = match_instance_param(localinst, key))
+		!= NULL && (ips->type == XC_STRING)) {
+	    nextptr = ips->parameter.string;
+	    /* This could probably be done better. . . hack to keep  */
+	    /* free() from operating twice on the same memory block. */
+	    promote->data.string = (char *)malloc(1);
+	 }
+	 else {
+	    promote->data.string = evaluate_expr(ops, localinst);
+            nextptr = promote;
+	 }
+      }
    }
    else
       nextptr = ops->parameter.string;
@@ -1101,7 +1159,8 @@ stringpart *stringcopyall(stringpart *string, objinstptr thisinst)
 
 stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 {
-   stringpart *strptr, *newpart, *curend = NULL, *rettop, *curtop, *savend;
+   stringpart *strptr, *newpart, *curend = NULL;
+   stringpart *rettop, *curtop, *savend = NULL;
    char *key = NULL;
    oparamptr pparam;
    Boolean need_free;
@@ -1328,7 +1387,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
        pos = 0;
        for (strptr = drawlabel->string; strptr != NULL;
-		strptr = nextstringpart(strptr, localinst)) {
+		strptr = nextstringpartnorecompute(strptr, localinst)) {
 
 	  /* All segments other than text cancel any	*/
 	  /* existing overline/underline in effect.	*/
